#!/bin/bash
source gpio.sh
# GPIO Hilfsdatei einfuegen

# Konstanten
# Verzoegerung zwischen LED leuchten 
DELAY_TIME_LED=0.25
# Verzoegerung bei Switch Eingabe
DELAY_TIME_SWITCH=0.2
# Variablen
# correct=0 Eingaben in Ordnung, Spiel geht weiter
# correct=-1 Eingabe falsch, Abbruch
correct=0

MakeRandom()
{ # Zufallszahl zwischen 0 und 3
  echo $((RANDOM%4))
}

LightLed()
{   # gibt alle Werte aus dem Array wieder, zeigt
    # also an welche Werte eingegeben werden
    # muessen um die Runde zu schaffen.
  # Alle Werte im Array abrufen
  for value in "${simonArray[@]}"
  do
    # gemaess dem Array Wert die pasende LED
    # an- und ausschalten
    case "$value" in
      0)
        ledNorthOn
        sleep ${DELAY_TIME_LED}
        ledNorthOff
        sleep ${DELAY_TIME_LED}
        ;;
      1)
        ledEastOn
        sleep ${DELAY_TIME_LED}
        ledEastOff
        sleep ${DELAY_TIME_LED}
        ;;
      2)
        ledWestOn
        sleep ${DELAY_TIME_LED}
        ledWestOff
        sleep ${DELAY_TIME_LED}
        ;;
      3)
        ledSouthOn
        sleep ${DELAY_TIME_LED}
        ledSouthOff
        sleep ${DELAY_TIME_LED}
        ;;
    esac
  done
}

ReadButton()
{ # Liest den Zustand der 4 Schalter ein
  # sollte einer gedrueckt sein wird die
  # Abfrage abgebrochen und der Wert
  # zurueck gegeben. Ist keiner gedrueckt,
  # wird -1 zurueck gegeben.
  if  [ $(switchNorth) = 1 ]
  then
    echo 0
  elif  [ $(switchEast) = 1 ]
  then
	 echo 1
  elif  [ $(switchWest) = 1 ]
  then
    echo 2
  elif  [ $(switchSouth) = 1 ]
  then
    echo 3
  else
    echo -1
  fi
}

CheckButton()
{   # Ueberprueft ob die eingegebene
    # Kombination der physikalischen
    # Schalter der vorgegebenen
    # entspricht. Falls ja, geht
    # das Spiel weiter, wenn nicht,
    # wird correct = -1 gesetzt.
  # Alle Werte im Array abrufen
  for value in "${simonArray[@]}"
  do
    # Wert der Schalter auslesen
    btn=$(ReadButton)
    # Solange kein Schalter gedrueckt
    # ist, weiter auslesen
    while [ $btn == -1 ]
    do
     btn=$(ReadButton)
    done
    # Falls ein Schalter gedrueckt wurde,
    # und der Wert der Stelle im Array
    # entspricht, weiter machen
    if [ $btn == $value ]
    then
      echo "Ok!"
      # sleep hilft an dieser Stelle einem
      # zu lange gedrueckten Schalter vorzubeugen.
      # Alternativ koennte man in ReadButton
      # einen Flankenwechsel als Schalterdruck
      # identifizieren
      sleep ${DELAY_TIME_SWITCH}
    # Sonst: correct=-1 setzen und
    # aus der Schleife ausbrechen
    else
      echo "Falsch!"
      correct=-1
      break
    fi
  done
}

# 2x schnelles blinken aller LEDs
# als "Willkommen"
ledBlinkFast
sleep 0.5

# Solange die eingegebenen Kombinationen richtig waren,
# geht das Spiel immer weiter.
while [ $correct == 0 ]
do
  # An der Stelle "groesse des Arrays" einen neuen Zufallswert
  # zwischen 0 und 3 in das Array einfuegen. Damit sparen
  # wir uns die Zahlvariable / Iterationsvariable
  simonArray[${#simonArray[@]}]=$(MakeRandom)
  # Alle Zufallswerte des Arrays auf den LEDs abspielen
  LightLed
  # Die Eingabe der Zufallswerte ueber die Schalter
  # erwarten und auf Fehler auswerten. Bei Fehler
  # wird correct=-1 gesetzt und das Spiel endet.
  # Sonst geht es endlos weiter.
  CheckButton
done

# 2x langsames blinken aller LEDs
# bei Fehler / Ende des Spiels
sleep 0.5
ledBlink

echo ${#simonArray[@]}
echo ${simonArray[@]}

# Neustart des Spiels bei Fehler
./simon.sh
