// wiringPi 2 MathLink

#include "mathlink.h"
#include <stdio.h>
#include <wiringPi.h>
// Include and Define I2C
#include <wiringPiI2C.h>
#include <stdlib.h>
#define lm75b_addr 0x48
int lm75b_file;
int n;
int lm75_temp;
signed char lm75b_hi;
signed char lm75b_lo;
float lm75b_temp;
// Include and Define SPI
#include <stdint.h>
#include <wiringPiSPI.h>

extern void setPinMode(int pin, int mode);
extern void setDigitalWrite(int pin, int value);
extern void setPwmWrite(int pin, int value);
extern int getDigitalRead(int pin);
extern float getLM75B();
extern int getMCP3002( int channel);

void setPinMode(int pin, int mode)
{
  pinMode(pin, mode);
  MLPutSymbol(stdlink, "Null");
}

void setDigitalWrite(int pin, int value)
{
  digitalWrite(pin, value);
  MLPutSymbol(stdlink, "Null");
}

void setPwmWrite(int pin, int value)
{
  pwmWrite(pin, value);
  MLPutSymbol(stdlink, "Null");
}

int getDigitalRead(int pin)
{
  return digitalRead(pin);
}

float getLM75B()
{
  if ((lm75b_file = wiringPiI2CSetup(lm75b_addr)) == -1)
  { 
    printf("  Kann den I2C Bus nicht oeffnen!\n");
    return -1;
  }
  // Verbindung zu LM75B oeffnen
  lm75_temp = wiringPiI2CReadReg16(lm75b_file, 0); 
  // Register 0 von LM75B in 16 Bit Tiefe auslesen    
  lm75b_hi = (lm75_temp & 0xFF);
  lm75b_lo = (lm75_temp >> 8);
  // In 2x 8 Bit Werte (High und Low Bit) umwandeln  
  n = lm75b_hi << 3 | ((lm75b_lo >> 5) & 0x07);
  /* gemaess Datasheet des Sensors aus den ausgelesenen
  2 Bytes die korreten 11 Bits für den Temperaturwert
  extrahieren und shiften */
  lm75b_temp = (float)n / 8;		
  // 11 Bit Wert in Temperatur umwandeln
  return lm75b_temp;
}

int getMCP3002(int channel)
{
  uint8_t buff[2];
  if ((channel > 1) || (channel < 0))
  {
    // Nur MCP Kanal 0 und 1 zulassen
    return -1;    
  }
  
  if (wiringPiSPISetup (0, 1000000) < 0)
  {
    return -1 ;
  }
  /* WiringPi SPI API initalisieren auf Slave Select / SPI Kanal 0 
  mit Geschwindigkeit 1 MHz */
  buff[0] = (6+channel)<<4;
  buff[1] = 0;
  // korrekte Anfrage fuer MCP3002 gemaess Datenblatt erzeugen
  wiringPiSPIDataRW(0, buff, 2);
  // Anfrage absenden und Antwort in den Puffer schreiben
  return ((buff[0]&3) << 8) + buff[1];
  // Wert aus Puffer shiften
}


int main(int argc, char* argv[])
{
  wiringPiSetupGpio();
  return MLMain(argc, argv);
}

