#!/usr/bin/python
import wiringpi2 as wpi
from time import sleep

lm75b_addr=0x48
# I2C Adresse des LM75B
LEDs=[24,18,23]
# LED Pins als Array fuer
# blau, gruen, rot
modifierState=[-1,0,+1]
# Temperatur Unterschiede fuer Ubergang
nameState=["Kalt","Normal","Warm"]
# Namen der States
currentState=0
# State der Pseudo State Machine
maxStates=3
# Anzahl der States

def readTemp():
  lm75_temp = wpi.wiringPiI2CReadReg16(lm75b_file, 0)
  # Register 0 von LM75B in 16 Bit Tiefe auslesen
  lm75b_hi = (lm75_temp & 0xFF)
  lm75b_lo = (lm75_temp >> 8)
  # In 2x 8 Bit Werte (High und Low Bit) umwandeln 
  n = lm75b_hi << 3 | ((lm75b_lo >> 5) & 0x07)
  # gemaess Datasheet des Sensors aus den ausgelesenen
  # 2 Bytes die korreten 11 Bits fuer den Temperaturwert    
  # extrahieren und shiften
  lm75b_temp = n / 8.0		
  # 11 Bit Wert in Temperatur umwandeln
  return lm75b_temp

def transition(up):
  global currentState, maxMod, minMod
  # globale Variablen in Funktion laden
  wpi.digitalWrite(LEDs[currentState],0)
  # Led des jetzigen State abschalten
  if (up==1):
  # Falls State erhoeht wird
    currentState=currentState+1
    # State erhoehen
    print("Transition von %s -> %s" % (nameState[currentState-1], nameState[currentState]))
    # Ausgabe vorheriger State nach jetziger State
  else:
    currentState=currentState-1
    # State verringern
    print("Transition von %s -> %s" % (nameState[currentState+1], nameState[currentState]))
    # Ausgabe vorheriger State nach jetziger State
  wpi.digitalWrite(LEDs[currentState],1)
  # Led des nun aktuellen State einschalten
  if (modifierState[currentState]==0):
  # Falls State 1 / Normal aktiv
    maxMod=1
    # setze maximalen Wert auf +1    
    minMod=-1
    # setze minimalen Wert auf -1
  else:
  # sonst setze Max/Min gemaess Array
    maxMod=modifierState[currentState]
    minMod=modifierState[currentState]

lm75b_file = wpi.wiringPiI2CSetup(lm75b_addr)
# Verbindung zu LM75B oeffnen
if lm75b_file < 0:
  print("Kann den I2C Bus nicht oeffnen!")
else:
 # Setup
  wpi.wiringPiSetupGpio()
  # Initialisiert die Bibliothek unter Nutzung des
  # BCM GPIO Namenschema
  for led_pin in LEDs:
      wpi.pinMode(led_pin,1)
      wpi.digitalWrite(led_pin,0)
  # Alle Pins mit LEDs als Ausgabe setzen und abschalten
  idealTemp = readTemp()
  # Ideal Temperatur messen
  transition(1)
  # Initalisieren auf Normal Modus
                       
  while True:
    if ((readTemp() > (idealTemp+maxMod)) and (currentState < (maxStates-1))):
    # falls aktuelle Temperatur groesser als Ideal Temperatur + Modifier
    # und aktueller State kleiner als die maximale Anzahl an States
      transition(1)   
      # State erhoehen
    elif ((readTemp() < (idealTemp+minMod)) and (currentState > 0)):
    # falls aktuelle Temperatur kleiner als Ideal Temperatur + Modifier
    # und aktueller State groesser als 0
      transition(0)  
      # State verringern
    else:
    # falls innerhalb der Temperatur des States
      print("%s: %4.2f" % (nameState[currentState], readTemp()))
      # aktueller State und aktuelle Temperaturmessung ausgeben
    sleep(0.25)	
 
