/*
 * boardTest.c:
 *  Programm zum halbautomatischen Test 
 *  des Erweiterungsboards und Beispiel
 *  zur Nutzung der GPIOs mit wiringPi2 und C 
 *  Testet 
 *  - LEDs
 *  - PWM
 *  - Schalter
 *  - I2C LM75B
 *  - SPI MCP3002  
 */

// Include wiringPi
#include <stdio.h>
#include <wiringPi.h>
// Include and Define I2C
#include <wiringPiI2C.h>
#include <stdlib.h>
#define lm75b_addr 0x48
int lm75b_file;
int n;
int lm75_temp;
signed char lm75b_hi;
signed char lm75b_lo;
float lm75b_temp;
// Include and Define SPI
#include <stdint.h>
#include <wiringPiSPI.h>
#define ADCCHANNEL 0
// LED Pins
#define	LEDNORTH	18
#define LEDEAST   23
#define LEDWEST   25
#define LEDSOUTH  24
// Switch Pins
#define SWITCHNORTH 4
#define SWITCHEAST  17
#define SWITCHWEST  22
#define SWITCHSOUTH 27
// PWM Variable
int bright;
int i;

int main (void)
{
  wiringPiSetupGpio();
  // wiringPi mit GPIO Bennungschema intialisieren
  /*
  *  LED Test
  */ 
  pinMode (LEDNORTH, OUTPUT);
  pinMode (LEDEAST, OUTPUT);
  pinMode (LEDWEST, OUTPUT);
  pinMode (LEDSOUTH, OUTPUT);
  // LED Pins als Ausgabe definieren
    
  printf ("LED Test beginnt\n") ;
  delay(750);

  printf ("  NORTH: LED gruen\n") ;
  digitalWrite(LEDNORTH, HIGH);
  // LED Pin North aktivieren
  delay(750);

  printf ("  EAST: LED rot\n") ;
  digitalWrite(LEDEAST, HIGH);
  delay(750);
  
  printf ("  WEST: LED gelb\n") ;
  digitalWrite(LEDWEST, HIGH);
  delay(750);
  
  printf ("  SOUTH: LED blau\n") ;
  digitalWrite(LEDSOUTH, HIGH);
  delay(2000);    

  digitalWrite(LEDNORTH, LOW);
  digitalWrite(LEDEAST, LOW);
  digitalWrite(LEDWEST, LOW);
  digitalWrite(LEDSOUTH, LOW);
  // LED Pins deaktivieren

  printf ("LED Test beendet\n");

  /*
  *  PWM Test
  */  
  pinMode (LEDNORTH, PWM_OUTPUT);
  // LED Pin North als PWM Ausgang definieren
  
  printf ("PWM Test beginnt\n");
  
  for (i=0; i < 4; i++)
  {
    for (bright = 0 ; bright < 1024 ; ++bright)
    {
      pwmWrite (LEDNORTH, bright);
      // LED Pin mit PWM Wert ansteuern.
      delay (1) ;
    }

    for (bright = 1023 ; bright >= 0 ; --bright)
    {
      pwmWrite (LEDNORTH, bright) ;
      delay (1) ;
    }
  }
  printf ("PWM Test beendet\n");

  /*
  *  Switch Test
  */      
  pinMode (SWITCHNORTH, INPUT);
  pinMode (SWITCHEAST, INPUT);
  pinMode (SWITCHWEST, INPUT);
  pinMode (SWITCHSOUTH, INPUT);
  // Switch Pins als Eingabe definieren
  printf ("Switch Test beginnt\n") ;

  printf ("  NORTH: Switch druecken\n") ;  
  while (digitalRead(SWITCHNORTH) == LOW)
  // So lange der Switch North nicht gedrückt ist, warten
  delay(1);
  printf ("  NORTH: Switch ok\n") ;

  printf ("  EAST: Switch druecken\n") ;  
  while (digitalRead(SWITCHEAST) == LOW)
  delay(1);
  printf ("  EAST: Switch ok\n") ;

  printf ("  WEST: Switch druecken\n") ;  
  while (digitalRead(SWITCHWEST) == LOW)
  delay(1);
  printf ("  WEST: Switch ok\n") ;

  printf ("  SOUTH: Switch druecken\n") ;  
  while (digitalRead(SWITCHSOUTH) == LOW)
  delay(1);
  printf ("  SOUTH: Switch ok\n") ;

  printf ("Switch Test beendet\n") ;

  /*
  *  LM75B Sensortest
  *  Original von http://www.keesmoerman.nl/raspberry.html
  *  Fehlerkorrektur für falschen Bus von Nico Maas    
  */     
  printf ("LM75B Temperatursensor Test beginnt\n") ;
  if ((lm75b_file = wiringPiI2CSetup(lm75b_addr)) == -1)
  { 
    printf("  Kann den I2C Bus nicht oeffnen!\n");
    return -1;
  }
  // Verbindung zu LM75B oeffnen

  for (i=0; i < 10; i++)
  {
    lm75_temp = wiringPiI2CReadReg16(lm75b_file, 0); 
    // Register 0 von LM75B in 16 Bit Tiefe auslesen    
    lm75b_hi = (lm75_temp & 0xFF);
    lm75b_lo = (lm75_temp >> 8);
    // In 2x 8 Bit Werte (High und Low Bit) umwandeln  
    n = lm75b_hi << 3 | ((lm75b_lo >> 5) & 0x07);
    /* gemaess Datasheet des Sensors aus den ausgelesenen
    2 Bytes die korreten 11 Bits für den Temperaturwert
    extrahieren und shiften */
    lm75b_temp = (float)n / 8;		
    // 11 Bit Wert in Temperatur umwandeln
    printf("  LM75B: %4.2f Grad Celsius\n", lm75b_temp);
    // Temperatur mit 2 Nachkommastellen ausgeben	
   
    delay(750);
   }   
  printf ("LM75B Temperatursensor Test beendet\n") ;

  /*
  *  MCP3002 ADC Test
  *  Angepasst von http://raspberrypihobbyist.blogspot.de/2012/12/analog-interface.html  
  */      
  printf ("MCP3002 ADC Test beginnt\n") ;
  // SPI Daten von MCP3002 Chip lesen, 2 moegliche Kanaele (0 und 1)
  int readadc(adcnum)
  {
    uint8_t buff[2];
    if ((adcnum > 1) || (adcnum < 0))
    {
      return -1;    
    }
    // Nur MCP Kanal 0 und 1 zulassen
    buff[0] = (6+adcnum)<<4;
    buff[1] = 0;
    // korrekte Anfrage fuer MCP3002 gemaess Datenblatt erzeugen
    wiringPiSPIDataRW(0, buff, 2);
    // Anfrage absenden und Antwort in den Puffer schreiben
    return ((buff[0]&3) << 8) + buff[1];
    // Wert aus Puffer shiften
  }

  if (wiringPiSPISetup (0, 1000000) < 0)
  {
    return -1 ;
  }
  /* WiringPi SPI API initalisieren auf Slave Select / SPI Kanal 0 
  mit Geschwindigkeit 1 MHz */
    
  for (i=0; i < 10; i++)
  {
    // Wert von MCP3002, Kanal 0 (LDR) ausgeben
    printf("  MCP3002 - Channel 0:  %d \n", readadc(ADCCHANNEL));
    delay(750);
  }
  printf ("MCP3002 ADC Test beendet\n") ;
  

  return 0 ;
}
